
# argsparse section
import sys, os
sys.path.append(os.getcwd())

from TextClustering.argsparse_clustering_preamble import argsparse_preamble
args = argsparse_preamble()

from sklearn.feature_extraction.text import TfidfVectorizer
import matplotlib.pyplot as plt
import pandas as pd
import umap
import hdbscan
import numpy as np
from nltk import RegexpTokenizer
from TextClustering.utils_metrics import ClusterMetrics
from database_preparation.preprocess import print_meta_data
from TextClassification.classification_for_cluster_evaluation import cross_validate_label_corpus_with_simple_SVM
import json

tokenizer = RegexpTokenizer(r'\w+')

# load the data
'''with open(args.path2corpus, 'rb') as f:
    doc_list = pickle.load(f)'''


assert args.path2corpus[-4:] == '.pkl'
corpus_df = pd.read_pickle(args.path2corpus)
corpus_dict = {case_id: corpus_df.loc[corpus_df['case_id'] == case_id, 'preprocessed_text'].values[0] for case_id in corpus_df['case_id']}
doc_list = corpus_df['preprocessed_text'].tolist()


#print_meta_data(args.path2corpus)

def identity(word):
    return word

text_is_vectorized = False #is_text_lst_tfidf_vectorized(args.path2corpus)
if not type(doc_list[0]) == list:
    print("Error: " + args.path2corpus + ' has to be a list of tokenized texts or tfidf-vectorized texts!')
    exit(1)

def create_vectorizer(data):
    vec = TfidfVectorizer(tokenizer=identity, preprocessor=None, lowercase=False)
    vec = vec.fit_transform(data)
    return vec

if text_is_vectorized:
    text_features = doc_list
else:
    text_features = create_vectorizer(doc_list)

print(f"tf-idf vectorized corpus generated {text_features.shape[0]} vectors of length {text_features.shape[1]}.")

# perform umap for dimension-reduction (for cluster-detection)
umap_text_features = umap.UMAP(n_neighbors=15,
                            n_components=5,
                            metric='cosine').fit_transform(text_features)

# and perform umap-dimension-reduction for visulatizaton
umap_text_features2D = umap.UMAP(n_neighbors=15,
                                 n_components=2,
                                 min_dist=0.0, metric='cosine').fit_transform(text_features)

if args.find_k_value:
    #  perform hdbscan for cluster-dectection with different cluster sizes to find a good solution...by eye..
    list_cluster_size = [int(k) for k in np.arange(3, 23, 1)]
    s_score, n_cluster, svm_scores = [], [], []
    for i_cluster_size in list_cluster_size:

        cluster = hdbscan.HDBSCAN(min_cluster_size=i_cluster_size,
                                  metric='euclidean',
                                  cluster_selection_method='eom').fit(umap_text_features)


        df_clustering_result2d = pd.DataFrame(umap_text_features2D, columns=['x', 'y'])
        df_clustering_result2d['labels'] = cluster.labels_.tolist() # cluster.labels_

        print(np.unique(df_clustering_result2d.labels))

        # Visualize clusters
        outliers = df_clustering_result2d.loc[df_clustering_result2d.labels == -1, :]
        clustered = df_clustering_result2d.loc[df_clustering_result2d.labels != -1, :]
        clustered['labels'] = [str(i) for i in clustered['labels']]

        evaluation = ClusterMetrics(umap_text_features, cluster.labels_.tolist())
        s_score.append(evaluation.s_score)
        svm_scores.append(
            cross_validate_label_corpus_with_simple_SVM(cluster.labels_.tolist(), args.path2corpus,
                                                        False))
        n_cluster.append(len(np.unique(cluster.labels_.tolist())))

    fig, ax1 = plt.subplots()
    ax2 = ax1.twinx()
    ax3 = ax1.twinx()
    ax1.plot(list_cluster_size, s_score, 'bx-')
    ax2.plot(list_cluster_size, n_cluster, 'rx-')
    ax3.plot(list_cluster_size, svm_scores, 'gx-')
    ax1.set_xlabel('Minimal cluster size')
    ax1.yaxis.label.set_color('blue')
    ax1.set_ylabel('s-score')
    ax2.yaxis.label.set_color('red')
    ax2.set_ylabel('Number of clusters')
    ax3.yaxis.label.set_color('green')
    ax3.set_ylabel('svm accuracy')
    plt.title('Ellbow-method-like plot')
    plt.savefig("TextClustering/plots/elbow_method/HDBSCAN_elbow_plot.png", dpi=300)
    plt.show()

    exit()

# perform hdbscan with best cluster size
cluster = hdbscan.HDBSCAN(min_cluster_size=args.k_value,
                              metric='euclidean',
                              cluster_selection_method='eom').fit(umap_text_features)


df_clustering_result2d = pd.DataFrame(umap_text_features2D, columns=['x', 'y'])
df_clustering_result2d['labels'] = cluster.labels_.tolist() # cluster.labels_
clusters = np.int8([str(i) for i in df_clustering_result2d['labels']])
outliers = df_clustering_result2d.loc[df_clustering_result2d.labels == -1, :]
clusters_no_outliers = df_clustering_result2d.loc[df_clustering_result2d.labels != -1, :]

unique_clusters = np.unique(df_clustering_result2d.labels)

# save umaped vectors:
#df = pd.read_pickle(args.df_cases_file)
corpus_df['umapX_HDBSCAN'] = umap_text_features2D[:, 0]
corpus_df['umapY_HDBSCAN'] = umap_text_features2D[:, 1]
corpus_df['label_HDBSCAN'] = clusters
corpus_df.to_pickle(args.path2corpus)
corpus_df.to_csv(args.path2corpus.replace('.pkl', '.csv'))
#corpus_df.to_excel(args.path2corpus.replace('.pkl', '.xlsx'))

print(f"updated {args.path2corpus} with umapX_HDBSCAN, umapY_HDBSCAN and label_HDBSCAN.")
print(corpus_df.head())

print(f"cluster count: {len(unique_clusters)}")
print(f"outliers: {len(outliers)}")

# and evaluate the results with several metrics (not needing ground truth)
evaluation = ClusterMetrics(umap_text_features[df_clustering_result2d.labels >= 0,], clusters_no_outliers.labels.tolist(),
                            file_name=args.path2corpus.replace('.pkl', '').replace('.json', '').replace('.df', '') + '_HDBSCAN_metrics')
evaluation.write_to_file()
