from TextClustering.utils_wordlist import generate_save_topicwords
import pandas as pd
from database_preparation.utils_labeled_datasets import label_list_as_int_list
from database_preparation.preprocess import get_metadata
import pickle
import openpyxl
import os, sys

argv = sys.argv

# parameters:
df_cases_file = "data/bow_short_diag/bow_short_diag.df.pkl"
df_cases_file = argv[1]
print_latex = False
filter_stop_words = True
path2umap_pics = 'TextClustering/plots/UMAP/'
save_umap_picture_in_table = True
stopword_filtered_corpus = True

####### functions ##########


def main():
    cluster_sets = ['HDBSCAN'] #['HDBSCAN', 'German_BERT', 'Patho_BERT', 'top2vec', 'GSDPMM']
    # cluster_sets = ['German_BERT']

    df_cases = pd.read_pickle(df_cases_file)

    for cluster_set in cluster_sets:
        # re-generate the topic words:
        excel_file_path = df_cases_file.replace('.df.pkl', f'_{cluster_set}_topwords.xlsx') #'TextClustering/tables/WordsPerCluster_' + cluster_set + '.xlsx'

        # convert nan-values in int(-1):
        try:
            clusters = label_list_as_int_list(df_cases['label_' + cluster_set])
        except:
            print(f"skipping {cluster_set}. it is not in the df_cases_file.")
            continue

        report_list = df_cases['preprocessed_text'].tolist()

        # do not apply stopwordfilterg if it is already stopword filtered!
        generate_save_topicwords(clusters, report_list, save_excel_file_path=excel_file_path,
                                 n_words=10, print_latex_table=print_latex,
                                 filter_stop_words=filter_stop_words and not stopword_filtered_corpus)
        if save_umap_picture_in_table:
            pic_path = path2umap_pics + cluster_set + "_UMAP.png"
            try:
                img = openpyxl.drawing.image.Image(pic_path)
                wb = openpyxl.load_workbook(excel_file_path)
                ws = wb.create_sheet("umap")
                img.anchor = 'A1'
                img.width = img.width / 2
                img.height = img.height / 2
                ws.add_image(img)
                wb.save(excel_file_path)
                print(f"Generated {excel_file_path}")
            except:
                print("could not load " + pic_path)
                print("therefore, cant place umap picture into " + excel_file_path)


    ######### topic words of authors #########
'''    clusters = label_list_as_int_list(df_cases['label_author'])
    excel_file_path = 'TextClustering/tables/WordsPerCluster_authors.xlsx'
    with open(path2corpus_bow_preprocessed, 'rb') as f:
        report_list = pickle.load(f)

    generate_save_topicwords(clusters, report_list, save_excel_file_path=excel_file_path,
                             n_words=20, print_latex_table=print_latex,
                             filter_stop_words=False)'''


if __name__ == '__main__':
    main()
